<?php
// WARNING: there must not be any empty lines before the php open tag

/***************************************************************************
 * calendar.php, version 0.3.3                                             *
 * This is a calendar display macro for the WikkiTikkiTavi wiki engine     *
 * Copyright (c) 2002,2003 Stephan Uhlmann <su@su2.info>                   *
 * Contributions by: Hermann Schwarting <hfs@gmx.de>                       *
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// =========================================================================
// Configuration
// =========================================================================

// This configures the style how weekdays are presented.
// 0 - american style, Sun to Sat
// 1 - european style, Mon to Sun
$calendar_weekstyle=1;

// This configures the style how dates are presented.
// 0 - american style, 11/17/2002
// 1 - european style, 17.11.2002
$calendar_datestyle=1;

// How many months relative to the current one to start the calendar.
// Can be a negative (going back) and a positive (going ahead) number.
// Is the value zero (0) the calendar will start with the current month.
$calendar_months_start=-1;

// How many months altogether to show at the calendar.
$calendar_months_number=12;

// How many months after we should wrap and start a new row.
$calendar_month_wrap=4;

// Whether to display upcoming events (true) or not (false) at the bottom of
// the calendar.
$calendar_display_events=true;

// This is the prefix of the pages linking to the dates.
$calendar_page_prefix="TerminPlaner";

// You can set a locale here to use other languages.
// setlocale (LC_ALL, 'de_DE');

/*

You can define different colours for the dates with or without an event in the
template/wiki.css file. The class attributes for the TD tag are:

calendar-today-event   : current date and there is an event
calendar-today-noevent : current date but there is no event
calendar-event         : any other date with an event
calendar-noevent       : any other date without an event


Example:

td.calendar-today-event
  { background-color: #ff3333;
    color:            #97694f;
    text-align:       center; }
td.calendar-today-noevent
  { background-color: #ffc3c3;
    color:            #97694f;
    text-align:       center; }
td.calendar-event
  { background-color: #c3ffc3;
    color:            #97694f;
    text-align:       center; }
td.calendar-noevent
  { color:            #97694f;
    text-align:       center; }

*/

// ======================================================================


function view_macro_calendar()
{
	global $calendar_months_start, $calendar_months_number, $calendar_month_wrap, $calendar_display_events;


	$r="";

	// start configured number of months before/ahead
	$itime=mktime(0,0,0,date("m"),16,date("Y"))+$calendar_months_start*2592000;

	$r.="<table border=\"0\">\n";
	$r.="<tr>\n";

	$i=0;

	while ($i<$calendar_months_number)
	{
		$r.="<td valign=\"top\">\n";
		$r.=calendar_month(strftime("%m",$itime),strftime("%Y",$itime));
		$r.="</td>\n";
		// +1 month
		// It's actually 30 days which could break when displaying a lot of months
		// but it should be ok when displaying only one or two years at the same time
		$itime=$itime+2592000;
		$i++;
		if (($i%$calendar_month_wrap==0) && ($i!=12)) $r.="</tr>\n<tr>";
	}

	$r.="</tr>\n";
	$r.="</table>\n";

	if ($calendar_display_events==true)
	{
		$r.="<br /><br />";
		$r.=next_events();
	}

	return $r;
}

function calendar_month($month,$year)
{
	global $pagestore, $PgTbl;
	global $calendar_page_prefix, $calendar_weekstyle;

	$r="";

	$todays_time=mktime(0,0,0,date("m"),date("d"),date("Y"));
	$last_day_of_month=strftime("%d",mktime(0,0,0,$month+1,0,$year));

	$r.="<table border=\"1\">\n";

	// header with month and year
	$r.="<tr><td colspan=\"7\">". strftime("%B",mktime(0,0,0,$month,1,$year)) . " ". $year . "</td></tr>\n";

	// weekday names
	$r.="<tr>";
	for ($i=$calendar_weekstyle;$i<7+$calendar_weekstyle;$i++)
	{
		$r.="<td>" . strftime("%a",259200+($i*60*60*24)) . "</td>";
	}
	$r.="</tr>\n";

	$count=0;

	// pre-padding
	$r.="<tr>";
	for ($i=0;$i<(strftime("%w",mktime(0,0,0,$month,1,$year))-$calendar_weekstyle+7)%7;$i++)
	{
		$r.="<td>&nbsp;</td>";
		$count++;
	}

	// days
	for ($i=1;$i<=$last_day_of_month;$i++)
	{

		if (($count%7==0) && ($count!=0)) $r.="</tr>\n<tr>";

		if ($i<10) { $day="0".$i; } else { $day=$i; }
		$page=$calendar_page_prefix . "/" . $year . $month . $day;
		// Only select pages which match currently processed date, have a current
		// version and a non-empty body description
		// The "time=supercede" is simpler than any SQL-hack to get MAX(version),
		// but it can break when Tavi developers change that logic.
		$query=$pagestore->dbh->query("SELECT COUNT(*) FROM $PgTbl WHERE title=\"$page\" AND time=supercede AND LENGTH(body)>0");
		$result=$pagestore->dbh->result($query);

		$r.="<td class=\"calendar-";
		if (abs($todays_time - mktime(0,0,0,$month,$i,$year)) < 86400)
		{
			$r.="today-";
		}
		if ($result[0]>0)
		{
			$r.="event\">";
			$r.="<b>";
			$r.="<a href=\"".viewURL($page)."\">$day</a>";
			$r.="</b>";
		}
		else
		{
			$r.="noevent\">";
			$r.="<a href=\"".editURL($page)."\">$day</a>";
		}
		$r.="</td>";

		$count++;

	}

	// post-padding
	while ($count%7!=0)
	{
		$r.="<td>&nbsp;</td>";
		$count++;
	}

	$r.="</tr>\n";

	$r.="</table>\n";

	return $r;
}

function next_events()
{
	global $pagestore, $PgTbl;
	global $calendar_page_prefix, $calendar_datestyle;

	$r="";

	$todays_time=time();

	$daystr=strftime("%Y%m%d",$todays_time);
	// Only select pages which are the current version, have the correct prefix,
	// the event is not in the past and the body description is not empty.
	// The "time=supercede" is simpler than any SQL-hack to get MAX(version),
	// but it can break when Tavi developers change that logic.
	$query=$pagestore->dbh->query("SELECT title,body FROM $PgTbl WHERE time=supercede AND title LIKE '$calendar_page_prefix/%' AND STRCMP(title,'$calendar_page_prefix/$daystr')>=0 AND LENGTH(body)>0 ORDER BY title ASC");
	while ($result=$pagestore->dbh->result($query))
	{
		$year=substr($result[0],strlen($calendar_page_prefix)+1,4);
		$month=substr($result[0],strlen($calendar_page_prefix)+5,2);
		$day=substr($result[0],strlen($calendar_page_prefix)+7,2);
		// better check here than via a cryptic regex in SQL
		if (is_numeric($year) && is_numeric($month) && is_numeric($day))
		{
			$itime=mktime(23,59,59,$month,$day,$year);
			// $itime < $todays_time shouldn't happen because we compared strings
			// already in SQL, but it does not harm to check twice
			if ($itime > $todays_time)
			{
				$page = "$calendar_page_prefix/". $year . $month . $day;
				if ($calendar_datestyle==0) $r.= "<a href=\"".viewURL($page)."\">$month/$day/$year</a>";
					else $r.= "<a href=\"".viewURL($page)."\">$day.$month.$year</a>";
				$r.= " - ";
				if (strlen($result[1])>65) $r.=substr($result[1],0,65) . " ...";
					else $r.=$result[1];
				$r.= "<br />\n";
			}
		}
	}

	return $r;
}

// WARNING: there must not be any empty lines after the php close tag
?>
